/*
 *  dailylog.c - This program reads all the log registers on a Morningstar TriStar PWM, stores the latest in dailylog.txt, and produces dailylog.html.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` dailylog.c -o dailylog
 
 Run this program once a day after the sun has set but before midnight using a cron file with these lines.  Store the file at /etc/cron.d/dailylog.
 
 # Update the TriStar PWM daily log and web page
 
 59 23 * * * root /home/tom/powersystem/bin/dailylog			# Path to executable file - you need to configure this appropriately in the cron file
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

void writehtmlfile(char *logfilename, char *htmlfilename);

int main(void)
{
	FILE *outfile;
	modbus_t *ctx;
	int i, j, k, n, low, rc, indx[90];
	unsigned int hm, hourmeter[90], Logger_flags[90], alarm_daily[90];
	float Vb_min_daily[90], Vb_max_daily[90], Ah_daily[90];
	unsigned short Fault_daily[90], Tb_min_daily[90], Tb_max_daily[90], T_ab_daily[90], T_eq_daily[90], T_fl_daily[90];
	unsigned short data[16];
	time_t lclTime;
	struct tm *now;
	char tsdate[32], filepath[64], logfilename[64], htmlfilename[64];
	
	/* Initialize index array with ascending values starting at 0 */
	for (i=0;i<90;i++) indx[i]=i;
	
	/* Get current local time */
	lclTime = time(NULL);
	now = localtime(&lclTime);
	
	// Create time stamps for log results and file names
	strftime(tsdate, 32, "%m/%d/%Y", now);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%%Y/%%Ydailylog.txt",LOGFILEPATH);
	strftime(logfilename, 64, filepath, now);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%%Y/%%Ydailylog.html",WEBPAGEFILEPATH);
	strftime(htmlfilename, 64, filepath, now);
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed TRISTARPWM: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	usleep(2500);						// Give the charge controller time before requesting the first set of log registers
	
	j=0;
	for(i=0x8000; i<0x8600; i+=0x0010) {
		
		/* Read the log registers and convert the results to their proper values */
		rc = modbus_read_registers(ctx, i, 12, data);
		if (rc == -1) {
			fprintf(stderr, "modbus_read_registers: %s\n", modbus_strerror(errno));
			return -1;
		}
		
		hm=data[0] + ((data[1] & 0x00FF) << 16);
		if (hm != 0x000000 && hm != 0xFFFFFF) {
			hourmeter[j]=hm;
			
			Logger_flags[j]=((data[2] & 0x00FF) << 8) + (data[1] >> 8);
			
			alarm_daily[j]=(data[3] << 8) + (data[2] >> 8);
			
			Vb_max_daily[j]=data[4]*100.0/32768.0;
			
			Vb_min_daily[j]=data[5]*100.0/32768.0;
			
			Ah_daily[j]=data[6]*0.1;
			
			Fault_daily[j]=data[7];
			
			Tb_min_daily[j]=(data[8] & 0x00FF);
			
			Tb_max_daily[j]=(data[8] >> 8);
			
			T_ab_daily[j]=data[9];
			
			T_eq_daily[j]=data[10];
			
			T_fl_daily[j]=data[11];
			
			j++;
		}
		usleep(2500);					// Give the charge controller time before requesting next set of log registers
	}
	
	/* Close the MODBUS connection */
    modbus_close(ctx);
	
	/* Order the data with lowest hourmeter value first */
	n=j;
	low=0;
	for (i=0; i<n-1; i++) {
		if (hourmeter[i] > hourmeter[i+1]) {		//Search for the lowest hourmeter value
			low=i+1;
			break;
		}
	}
	if (low > 0) {									// Create an index to order the data
		for (i=0, j=n-low; i<low; i++, j++) indx[j]=i;
		for (i=low, j=0; i<n; i++, j++) indx[j]=i;
	}
	
	// Write data to log file
	if ((outfile = fopen(logfilename, "a")) == NULL) { 
		printf("Can't create log file.\n");
		exit(1);
	}
	
	// Write the last log record to log file (today's data)
	i=n-1;
	fprintf(outfile,"%s\t%u\t%u\t%.2f\t%.2f", tsdate, hourmeter[indx[i]],alarm_daily[indx[i]],Vb_min_daily[indx[i]],Vb_max_daily[indx[i]]);
	fprintf(outfile,"\t%.2f\t%d\t%d\t%d\t%d", Ah_daily[indx[i]],Fault_daily[indx[i]],Logger_flags[indx[i]],Tb_min_daily[indx[i]],Tb_max_daily[indx[i]]);
	fprintf(outfile,"\t%d\t%d\t%d\n", T_ab_daily[indx[i]],T_eq_daily[indx[i]],T_fl_daily[indx[i]]);
	
	fclose(outfile);
	
	writehtmlfile(logfilename, htmlfilename);
	
    modbus_free(ctx);
	
	return(0);
}

void writehtmlfile(char *logfilename, char *htmlfilename)
{
	FILE *htmlfile, *infile;
	int month, day, year;
	unsigned int hourmeter, alarm_daily;
	float Vb_min_daily, Vb_max_daily, Ah_daily;
	int Fault_daily, Logger_flags, Tb_min_daily, Tb_max_daily, T_ab_daily, T_eq_daily, T_fl_daily;
	char inputline[1000] = "";
	
	// Write data to html file
	if ((htmlfile = fopen(htmlfilename, "w")) == NULL) { 
		printf("Can't create file: %s.\n", htmlfilename);
		exit(1);
	}
	
	fprintf(htmlfile,"<html>\n<head>\n\t<title>TriStar PWM Daily Log</title>\n</head>\n");
	fprintf(htmlfile,"<body bgcolor=\"#6699FF\" text=\"#000000\" link=\"#330099\" vlink=\"#336633\" alink=\"#FFCC00\">\n");
	fprintf(htmlfile,"<h3 style=\"font-family:Comic Sans MS;color:#663300\">TriStar PWM Daily Log</h3>\n");
	fprintf(htmlfile,"<table border=\"1\" style=\"font-family:arial;color:black;font-size:12px;background-color:white;text-align:center;\">\n");
	fprintf(htmlfile,"\t<tr>\n\t\t<th>Date</th>\n");
	fprintf(htmlfile,"\t\t<th>Hour Meter</th>\n");
	fprintf(htmlfile,"\t\t<th>Min Vb</th>\n");
	fprintf(htmlfile,"\t\t<th>Max Vb</th>\n");
	fprintf(htmlfile,"\t\t<th>Daily Ah</th>\n");
	fprintf(htmlfile,"\t\t<th>Min Batt Temp</th>\n");
	fprintf(htmlfile,"\t\t<th>Max Batt Temp</th>\n");
	fprintf(htmlfile,"\t\t<th>Ab Time</th>\n");
	fprintf(htmlfile,"\t\t<th>Eq Time</th>\n");
	fprintf(htmlfile,"\t\t<th>Fl Time</th>\n");
	fprintf(htmlfile,"\t\t<th>Controller Alarms</th>\n");
	fprintf(htmlfile,"\t\t<th>Controller Faults</th>\n");
	fprintf(htmlfile,"\t\t<th>Logger Flags</th>\n\t</tr>\n");
	infile = fopen(logfilename, "r");
	if(infile != NULL) {
		while (fscanf(infile, "%[^\n]\n", inputline) != EOF)
		{
			sscanf(inputline,"%2d%*c%2d%*c%4d%u%u%f%f%f%d%d%d%d%d%d%d",&month,&day,&year,&hourmeter,&alarm_daily,&Vb_min_daily,
				   &Vb_max_daily,&Ah_daily,&Fault_daily,&Logger_flags,&Tb_min_daily,&Tb_max_daily,
				   &T_ab_daily,&T_eq_daily,&T_fl_daily);
			fprintf(htmlfile,"\t<tr>\n\t\t<td>%0d/%0d/%d</td>\n",month,day,year);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",hourmeter);
			fprintf(htmlfile,"\t\t<td>%.2f</td>\n",Vb_min_daily);
			fprintf(htmlfile,"\t\t<td>%.2f</td>\n",Vb_max_daily);
			fprintf(htmlfile,"\t\t<td>%.2f</td>\n",Ah_daily);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",Tb_min_daily);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",Tb_max_daily);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",T_ab_daily);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",T_eq_daily);
			fprintf(htmlfile,"\t\t<td>%d</td>\n",T_fl_daily);
			fprintf(htmlfile,"\t\t<td>");
			if (alarm_daily == 0) {
				fprintf(htmlfile,"No alarms</td>\n");
			} else {
				if (alarm_daily & 1) fprintf(htmlfile,"RTS open<br>");
				if ((alarm_daily & (1 << 1)) >> 1) fprintf(htmlfile,"RTS shorted<br>");
				if ((alarm_daily & (1 << 2)) >> 2) fprintf(htmlfile,"RTS disconnected<br>");
				if ((alarm_daily & (1 << 3)) >> 3) fprintf(htmlfile,"Ths disconnected<br>");
				if ((alarm_daily & (1 << 4)) >> 4) fprintf(htmlfile,"Ths shorted<br>");
				if ((alarm_daily & (1 << 5)) >> 5) fprintf(htmlfile,"TriStar hot<br>");
				if ((alarm_daily & (1 << 6)) >> 6) fprintf(htmlfile,"Current limit<br>");
				if ((alarm_daily & (1 << 7)) >> 7) fprintf(htmlfile,"Current offset<br>");
				if ((alarm_daily & (1 << 8)) >> 8) fprintf(htmlfile,"Battery Sense<br>");
				if ((alarm_daily & (1 << 9)) >> 9) fprintf(htmlfile,"Batt Sense disc<br>");
				if ((alarm_daily & (1 << 10)) >> 10) fprintf(htmlfile,"Uncalibrated<br>");
				if ((alarm_daily & (1 << 11)) >> 11) fprintf(htmlfile,"RTS miswire<br>");
				if ((alarm_daily & (1 << 12)) >> 12) fprintf(htmlfile,"HVD<br>");
				if ((alarm_daily & (1 << 13)) >> 13) fprintf(htmlfile,"high d<br>");
				if ((alarm_daily & (1 << 14)) >> 14) fprintf(htmlfile,"miswire<br>");
				if ((alarm_daily & (1 << 15)) >> 15) fprintf(htmlfile,"FET open<br>");
				if ((alarm_daily & (1 << 16)) >> 16) fprintf(htmlfile,"P12<br>");
				if ((alarm_daily & (1 << 17)) >> 17) fprintf(htmlfile,"Load Disc.<br>");
				if ((alarm_daily & (1 << 18)) >> 18) fprintf(htmlfile,"Alarm 19<br>");
				if ((alarm_daily & (1 << 19)) >> 19) fprintf(htmlfile,"Alarm 20<br>");
				if ((alarm_daily & (1 << 20)) >> 20) fprintf(htmlfile,"Alarm 21<br>");
				if ((alarm_daily & (1 << 21)) >> 21) fprintf(htmlfile,"Alarm 22<br>");
				if ((alarm_daily & (1 << 22)) >> 22) fprintf(htmlfile,"Alarm 23<br>");
				if ((alarm_daily & (1 << 23)) >> 23) fprintf(htmlfile,"Alarm 24");
				fprintf(htmlfile,"</td>\n");
			}
			fprintf(htmlfile,"\t\t<td>");
			if (Fault_daily == 0) {
				fprintf(htmlfile,"No faults</td>\n");
			} else {
				if (Fault_daily & 1) fprintf(htmlfile,"External Short<br>");
				if ((Fault_daily & (1 << 1)) >> 1) fprintf(htmlfile,"Overcurrent<br>");
				if ((Fault_daily & (1 << 2)) >> 2) fprintf(htmlfile,"FET short<br>");
				if ((Fault_daily & (1 << 3)) >> 3) fprintf(htmlfile,"Software<br>");
				if ((Fault_daily & (1 << 4)) >> 4) fprintf(htmlfile,"HVD<br>");
				if ((Fault_daily & (1 << 5)) >> 5) fprintf(htmlfile,"TriStar hot<br>");
				if ((Fault_daily & (1 << 6)) >> 6) fprintf(htmlfile,"DIP sw changed<br>");
				if ((Fault_daily & (1 << 7)) >> 7) fprintf(htmlfile,"Setting edit<br>");
				if ((Fault_daily & (1 << 8)) >> 8) fprintf(htmlfile,"reset?<br>");
				if ((Fault_daily & (1 << 9)) >> 9) fprintf(htmlfile,"Miswire<br>");
				if ((Fault_daily & (1 << 10)) >> 10) fprintf(htmlfile,"RTS shorted<br>");
				if ((Fault_daily & (1 << 11)) >> 11) fprintf(htmlfile,"RTS disconnected<br>");
				if ((Fault_daily & (1 << 12)) >> 12) fprintf(htmlfile,"Fault 12<br>");
				if ((Fault_daily & (1 << 13)) >> 13) fprintf(htmlfile,"Fault 13<br>");
				if ((Fault_daily & (1 << 14)) >> 14) fprintf(htmlfile,"Fault 14<br>");
				if ((Fault_daily & (1 << 15)) >> 15) fprintf(htmlfile,"Fault 15");
				fprintf(htmlfile,"</td>\n");
			}
			fprintf(htmlfile,"\t\t<td>");
			if (Logger_flags == 0) {		// The TriStar MODBUS document doesn't define the Logger flags. You may want to just print the value.
				fprintf(htmlfile,"No Logger Flags</td>\n\t</tr>\n");
			} else {
				if (Logger_flags & 1) fprintf(htmlfile,"Logger Flag 0\n");
				if ((Logger_flags & (1 << 1)) >> 1) fprintf(htmlfile,"Logger Flag 1<br>");
				if ((Logger_flags & (1 << 2)) >> 2) fprintf(htmlfile,"Logger Flag 2<br>");
				if ((Logger_flags & (1 << 3)) >> 3) fprintf(htmlfile,"Logger Flag 3<br>");
				if ((Logger_flags & (1 << 4)) >> 4) fprintf(htmlfile,"Logger Flag 4<br>");
				if ((Logger_flags & (1 << 5)) >> 5) fprintf(htmlfile,"Logger Flag 5<br>");
				if ((Logger_flags & (1 << 6)) >> 6) fprintf(htmlfile,"Logger Flag 6<br>");
				if ((Logger_flags & (1 << 7)) >> 7) fprintf(htmlfile,"Logger Flag 7<br>");
				if ((Logger_flags & (1 << 8)) >> 8) fprintf(htmlfile,"Logger Flag 8<br>");
				if ((Logger_flags & (1 << 9)) >> 9) fprintf(htmlfile,"Logger Flag 9<br>");
				if ((Logger_flags & (1 << 10)) >> 10) fprintf(htmlfile,"Logger Flag 10<br>");
				if ((Logger_flags & (1 << 11)) >> 11) fprintf(htmlfile,"Logger Flag 11<br>");
				if ((Logger_flags & (1 << 12)) >> 12) fprintf(htmlfile,"Logger Flag 12<br>");
				if ((Logger_flags & (1 << 13)) >> 13) fprintf(htmlfile,"Logger Flag 13<br>");
				if ((Logger_flags & (1 << 14)) >> 14) fprintf(htmlfile,"Logger Flag 14<br>");
				if ((Logger_flags & (1 << 15)) >> 15) fprintf(htmlfile,"Logger Flag 15<br>");
				fprintf(htmlfile,"</td>\n\t</tr>\n");
			}
		}
	}
	fclose(infile);
	
	fprintf(htmlfile,"</table>\n");
 	fprintf(htmlfile,"</body>\n</html>\n");
	
	fclose(htmlfile);
}

