/*
 *  powersystemstatus.c
 *  
 
 Copyright 2014 Tom Rinehart.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see http://www.gnu.org/licenses/.
 
 */

/* On Linux, compile with: cc `pkg-config --cflags --libs libmodbus` powersystemstatus.c -o powersystemstatus -lgd -lpng -lz */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>

#include <modbus.h>

#include "gd.h"
#include "gdfonts.h"
#include "gdfontl.h"

#include "powersystem.h"


void drawgraph(char *logfilename, char *graphfilename);
void drawpanelmeter(float number, char *label, char *filepath);
void plotdigit(gdImagePtr im, int digitValue, int digitLocation, int left, int top, int bordercolor, int fillcolor);
void plotdecimalpt(gdImagePtr im, int digitLocation, int left, int top, int bordercolor, int fillcolor);
void plotbase(gdImagePtr im, int x, int y, int color);
void plot0(gdImagePtr im, int x, int y, int color);
void plot1(gdImagePtr im, int x, int y, int color);
void plot2(gdImagePtr im, int x, int y, int color);
void plot3(gdImagePtr im, int x, int y, int color);
void plot4(gdImagePtr im, int x, int y, int color);
void plot5(gdImagePtr im, int x, int y, int color);
void plot6(gdImagePtr im, int x, int y, int color);
void plot7(gdImagePtr im, int x, int y, int color);
void plot8(gdImagePtr im, int x, int y, int color);
void plot9(gdImagePtr im, int x, int y, int color);
void plotminus(gdImagePtr im, int x, int y, int color);

int main(void)
{
	FILE *outfile, *htmlfile;
	time_t lclTime;
	struct tm *now;
	char ts[32], filepath[64], logfile[64], graphfilename[64], graphfilepath[64], tsdate[32], tstime[32];
	
	modbus_t *ctx;
	int rc;
	unsigned short charge_state;
	float tristar_Vb, tristar_Va, tristar_Ic;
	short tristar_Ths, tristar_Tb;
	char charge_state_string[32];
	uint16_t data[50];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the RAM Registers on the TriStar PWM and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0x0008, 22, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	/* Close the MODBUS connection */
	modbus_close(ctx);
	
//	tristar_Vb=data[0]*96.667/32768.0;		// adc_vb_f - Voltage measured directly at the battery connection on the TriStar
	tristar_Vb=data[1]*96.667/32768.0;		// adc_vs_f - Voltage reading at the Battery Sense terminals
//	tristar_Vb=data[5]*96.667/32768.0;		// Vb_f - Voltage measured directly at the battery connection on the TriStar - slow filtered
	tristar_Va=data[2]*139.15/32768.0;		// adc_vx_f - Voltage measured directly at the PV terminals
	tristar_Ic=data[3]*66.667/32768.0;		// adc_ipv_f - Slow filtered charging current
	tristar_Ths=data[6];					// T_hs - Heatsink temperature
	tristar_Tb=data[7];						// T_batt - Battery temperature
	charge_state=data[19];
	switch (charge_state) {
		case 0:
			strcpy(charge_state_string,"START");
			break;
		case 1:
			strcpy(charge_state_string,"NIGHT_CHECK");
			break;
		case 2:
			strcpy(charge_state_string,"DISCONNECT");
			break;
		case 3:
			strcpy(charge_state_string,"NIGHT");
			break;
		case 4:
			strcpy(charge_state_string,"FAULT");
			break;
		case 5:
			strcpy(charge_state_string,"BULK_CHARGE");
			break;
		case 6:
			strcpy(charge_state_string,"PWM");
			break;
		case 7:
			strcpy(charge_state_string,"FLOAT");
			break;
		case 8:
			strcpy(charge_state_string,"EQUALIZE");
			break;
	}
	
	
	/* Create a time stamps for data results, file names, and web page */
	lclTime = time(NULL);
	now = localtime(&lclTime);
	strftime(ts, 32, "%m/%d/%Y\t%H:%M", now);						// Time stamp for log file entries
	
	strcpy(filepath,"");
	sprintf(filepath,"%s/%%Y/%%Y%%m%%d.txt",LOGFILEPATH);			// File path (YYYY) and file name (YYYYMMDD.txt) for log file
	strftime(logfile, 64, filepath, now);							// You need to manually create the annual directory (YYYY) or write code to do this automatically
	
	strftime(graphfilename, 64, "%Y/%Y%m%d.png", now);				// File path (YYYY) and file name (YYYYMMDD.png) for daily graph image file
	strcpy(graphfilepath,"");										// You need to manually create the annual directory (YYYY) or write code to do this automatically
	sprintf(graphfilepath,"%s/%s",WEBPAGEFILEPATH,graphfilename);
	
	strftime(tsdate, 32, "%A, %B %d, %Y", now);						// Date stamp for web page updates
	strftime(tstime, 32, "%I:%M %p", now);							// Time stamp for web page updates
	
	/* Write data to log file */
	if ((outfile = fopen(logfile, "a")) == NULL) {
		printf("Can't create log file: %s\n", logfile);
		exit(1);
	}
	
	fprintf(outfile,"%s\t%5.2f\t%5.2f\t%5.2f\t%s\n", ts, tristar_Vb, tristar_Va, tristar_Ic, charge_state_string);
	
	fclose(outfile);
	
	/* Draw panel meter images for the TriStar PWM */
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/vbattery.png");	
	drawpanelmeter(tristar_Vb,"Battery Voltage",filepath);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/varray.png");	
	drawpanelmeter(tristar_Va,"Array Voltage",filepath);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/iarray.png");	
	drawpanelmeter(tristar_Ic,"Charging Current",filepath);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/chargepower.png");	
	drawpanelmeter(tristar_Vb*tristar_Ic,"Charging Power",filepath);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/hs_temp.png");	
	drawpanelmeter((float) tristar_Ths,"Heat Sink Temp.",filepath);
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,"panelmeters/batt_temp.png");	
	drawpanelmeter((float) tristar_Tb,"Battery Temp.",filepath);
	
	/* Draw the daily graph from the daily log file */
	drawgraph(logfile, graphfilepath);
	
	/* Write the html file to display the daily graph and the panel meter images */
	strcpy(filepath,"");
	sprintf(filepath,"%s/%s",WEBPAGEFILEPATH,MAINWEBPAGENAME);
	if ((htmlfile = fopen(filepath, "w")) == NULL) {
		printf("Can't create the html file: %s\n", filepath);
		exit(1);
	}
	
	fprintf(htmlfile,"<html>\n<head>\n<title>Power System Status</title>\n</head>\n");
	fprintf(htmlfile,"<body bgcolor=\"#6699FF\" text=\"#000000\" link=\"#330099\" vlink=\"#336633\" alink=\"#FFCC00\">\n");
	fprintf(htmlfile,"<font face=\"Comic Sans MS, Arial, Helvetica\">\n");
	fprintf(htmlfile,"<h3><font color=\"#663300\">Power System Status</font></h3>\n");
	fprintf(htmlfile,"<table>\n");
	
	/* Display the daily graph */
	fprintf(htmlfile,"<tr><td><img src=\"%s\"></td></tr>\n",graphfilename);
	fprintf(htmlfile,"<tr><td><table>\n");
	fprintf(htmlfile,"<tr>\n");
//	fprintf(htmlfile,"<td><a href=\"2016/2016dailygraphs.html\">2016 Daily Graphs</a></td>\n");			// You need to manually add a link for each year's graphs and manually
//	fprintf(htmlfile,"<td><a href=\"2015/2015dailygraphs.html\">2015 Daily Graphs</a></td>\n");			// create the annual directory or write code to do this automatically
	fprintf(htmlfile,"<td><a href=\"2014/2014dailygraphs.html\">2014 Daily Graphs</a></td>\n");
	fprintf(htmlfile,"</tr>\n");
	fprintf(htmlfile,"</table></td></tr>\n");
	
	/* Display the panel meters for the TriStar PWM */
	fprintf(htmlfile,"<tr><td><br><b>TriStar PWM</b><br><hr></td></tr>\n");
	fprintf(htmlfile,"<tr><td><table>\n");
	fprintf(htmlfile,"<tr><td><img src=\"panelmeters/vbattery.png\"></td><td>&nbsp;</td>");
	fprintf(htmlfile,"<td><img src=\"panelmeters/batt_temp.png\"></td><td><img src=\"panelmeters/hs_temp.png\"></td></tr>\n");
	fprintf(htmlfile,"<tr><td COLSPAN=\"4\">Charging State: %s</td></tr>\n",charge_state_string);
	fprintf(htmlfile,"<tr><td><img src=\"vvarray.png\"></td><td><img src=\"viarray.png\"></td>");
	fprintf(htmlfile,"<td>&nbsp;</td><td><img src=\"panelmeters/chargepower.png\"></td></tr>\n");
	fprintf(htmlfile,"</table></td></tr>\n");
	fprintf(htmlfile,"<tr><td><table>\n");
	fprintf(htmlfile,"<tr>\n");
//	fprintf(htmlfile,"<td><a href=\"2016/2016dailylog.html\">2016 Daily Log</a></td>\n");				// You need to manually add a link for each year's graphs and manually
//	fprintf(htmlfile,"<td><a href=\"2015/2015dailylog.html\">2015 Daily Log</a></td>\n");				// create the annual directory or write code to do this automatically
	fprintf(htmlfile,"<td><a href=\"2014/2014dailylog.html\">2014 Daily Log</a></td>\n");
	fprintf(htmlfile,"</tr>\n");				
	fprintf(htmlfile,"</table></td></tr>\n");
	fprintf(htmlfile,"</table>\n<br>\n");
 	fprintf(htmlfile,"<h6><font color=\"#663300\">Updated: <b>%s on %s</b></font></h6>\n",tstime,tsdate);
	fprintf(htmlfile,"</body>\n</html>\n");

	fclose(htmlfile);
	
	modbus_free(ctx);
	
	return(0);
}

void drawpanelmeter(float number, char *label, char *filepath)
{
	/* Declare the image */
	gdImagePtr im;
	/* Declare output files */
	FILE *pngout;
	/* Declare color indexes */
	int white, vltgrey, ltgrey, grey, dkgrey, black, red;
	/* Declare integers for each digit in the display */
	int d1, d2, d3, d4, sign;
	
	/* Allocate the image */
	im = gdImageCreate(112, 70);
 
	/* Allocate the color white (red, green and blue all maximum).
		Since this is the first color in a new image, it will
		be the background color. */
	white = gdImageColorAllocate(im, 255, 255, 255);
 
	/* Allocate the color black (red, green and blue all minimum). */
	black = gdImageColorAllocate(im, 0, 0, 0);
	
	/* Allocate other colors. */
	vltgrey = gdImageColorAllocate(im, 212, 212, 212);
	ltgrey = gdImageColorAllocate(im, 191, 191, 191);
	grey = gdImageColorAllocate(im, 127, 127, 127);
	dkgrey = gdImageColorAllocate(im, 63, 63, 63);
	red = gdImageColorAllocate(im, 255, 0, 0);
	
	sign=1;
	if (number < 0) {
		sign=-1;
		number*=-1.0;
	}
	
	if (number < 10.0 && sign<0) {
		d1=sign;
		d2=number;
		d3=number*10-d2*10;
		d4=number*100-d2*100-d3*10;
		plotdecimalpt(im, 2, 12, 12, vltgrey, red);
	}
	else if (number < 100.0 && sign<0) {
		d1=sign;
		d2=number/10;
		d3=number-d1*10;
		d4=number*10-d1*100-d2*10;
		plotdecimalpt(im, 3, 12, 12, vltgrey, red);
	}
	else if (number < 100.0) {
		d1=number/10;
		d2=number-d1*10;
		d3=number*10-d1*100-d2*10;
		d4=number*100-d1*1000-d2*100-d3*10;
		plotdecimalpt(im, 2, 12, 12, vltgrey, red);
	}
	else if (number < 1000.0) {
		d1=number/100;
		d2=(number-d1*100)/10;
		d3=number-d1*100-d2*10;
		d4=number*10-d1*1000-d2*100-d3*10;
		plotdecimalpt(im, 3, 12, 12, vltgrey, red);
	}
	else if (number < 10000.0) {
		d1=number/1000;
		d2=(number-d1*1000)/100;
		d3=(number-d1*1000-d2*100)/10;
		d4=number-d1*1000-d2*100-d3*10;
		plotdecimalpt(im, 4, 12, 12, vltgrey, red);
	}
	
	if (d1 != 0 )
		plotdigit(im, d1, 1, 12, 12, vltgrey, red);
	else
		plotbase(im, 12, 12, vltgrey);
	
	plotdigit(im, d2, 2, 12, 12, vltgrey, red);
	plotdigit(im, d3, 3, 12, 12, vltgrey, red);
	plotdigit(im, d4, 4, 12, 12, vltgrey, red);
	
	gdImageLine(im, 0, 0, 3, 3, vltgrey);
	gdImageLine(im, 5, 5, 6, 6, black);
	gdImageLine(im, 0, 55, 6, 49, grey);
	gdImageLine(im, 111, 0, 105, 6, grey);
	gdImageLine(im, 111, 55, 108, 52, black);
	gdImageLine(im, 106, 50, 105, 49, vltgrey);
	gdImageLine(im, 0, 56, 112, 56, black);
	gdImageRectangle(im, 4, 4, 107, 51, black);
	gdImageRectangle(im, 7, 7, 104, 48, black);
	gdImageFill(im, 0, 1, ltgrey);
	gdImageFill(im, 1, 0, ltgrey);
	gdImageFill(im, 111, 1, dkgrey);
	gdImageFill(im, 110, 55, dkgrey);
	gdImageFill(im, 5, 6, dkgrey);
	gdImageFill(im, 6, 5, dkgrey);
	gdImageFill(im, 105, 50, ltgrey);
	gdImageFill(im, 106, 49, ltgrey);
	gdImageFill(im, 1, 57, ltgrey);
	
	/* Draw panelmeter label in red */
	gdImageString(im, gdFontGetSmall(),im->sx / 2 - (strlen(label) * gdFontGetSmall()->w / 2), 56, label, red);

	/* Open a file for writing. "wb" means "write binary", important
		under MSDOS, harmless under Unix. */
	pngout = fopen(filepath, "wb");
	
	/* Output the image to the disk file in PNG format. */
	gdImagePng(im, pngout);
	
	/* Close the files. */
	fclose(pngout);
	
	/* Destroy the image in memory. */
	gdImageDestroy(im);
}

void plotdigit(gdImagePtr im, int digitValue, int digitLocation, int left, int top, int bordercolor, int fillcolor)
{
	plotbase(im, left+24*(digitLocation-1), top, bordercolor);
	
	if (digitValue < 0) {
		plotminus(im, left+24*(digitLocation-1), top, fillcolor);
	}
	else {
		switch (digitValue) {
			case 0:
				plot0(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 1:
				plot1(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 2:
				plot2(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 3:
				plot3(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 4:
				plot4(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 5:
				plot5(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 6:
				plot6(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 7:
				plot7(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 8:
				plot8(im, left+24*(digitLocation-1), top, fillcolor);
				break;
			case 9:
				plot9(im, left+24*(digitLocation-1), top, fillcolor);
				break;
		}
	}
}

void plotdecimalpt(gdImagePtr im, int digitLocation, int left, int top, int bordercolor, int fillcolor)
{
	/* Draw Decimal Point */
	int x, y;
	
	x = left+24*(digitLocation-1);
	y = top;
	
	gdImageLine(im, x+18, y+27, x+20, y+27, bordercolor);
	gdImageLine(im, x+17, y+28, x+17, y+30, bordercolor);
	gdImageLine(im, x+21, y+28, x+21, y+30, bordercolor);
	gdImageLine(im, x+18, y+31, x+20, y+31, bordercolor);
	gdImageFill(im, x+18, y+28, fillcolor);
}

void plotbase(gdImagePtr im, int x, int y, int color)
{
	/* Draw 7-Segment Base */
	gdImageLine(im, x+0, y+2, x+0, y+29, color);
	gdImageLine(im, x+15, y+2, x+15, y+29, color);
	gdImageLine(im, x+2, y+0, x+13, y+0, color);
	gdImageLine(im, x+2, y+31, x+13, y+31, color);
	gdImageLine(im, x+1, y+1, x+4, y+4, color);
	gdImageLine(im, x+14, y+1, x+11, y+4, color);
	gdImageLine(im, x+1, y+30, x+4, y+27, color);
	gdImageLine(im, x+14, y+30, x+11, y+27, color);
	gdImageLine(im, x+1, y+15, x+3, y+13, color);
	gdImageLine(im, x+1, y+15, x+3, y+17, color);
	gdImageLine(im, x+14, y+15, x+12, y+13, color);
	gdImageLine(im, x+14, y+15, x+12, y+17, color);
	gdImageRectangle(im, x+4, y+4, x+11, y+13, color);
	gdImageRectangle(im, x+4, y+17, x+11, y+27, color);
}

void plot0(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment E */
	gdImageFill(im, x+1, y+16, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
}

void plot1(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
}

void plot2(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment E */
	gdImageFill(im, x+1, y+16, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot3(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot4(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot5(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot6(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment E */
	gdImageFill(im, x+1, y+16, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot7(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
}

void plot8(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment E */
	gdImageFill(im, x+1, y+16, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plot9(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment A */
	gdImageFill(im, x+2, y+1, color);
	
	/* Fill Segment B */
	gdImageFill(im, x+14, y+2, color);
	
	/* Fill Segment C */
	gdImageFill(im, x+14, y+16, color);
	
	/* Fill Segment D */
	gdImageFill(im, x+2, y+30, color);
	
	/* Fill Segment F */
	gdImageFill(im, x+1, y+2, color);
	
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void plotminus(gdImagePtr im, int x, int y, int color)
{
	/* Fill Segment G */
	gdImageFill(im, x+2, y+15, color);
}

void drawgraph(char *logfilename, char *graphfilename) {
	/* Declare the image */
	gdImagePtr im;
	/* Declare output files */
	FILE *pngout, *infile;
	/* Declare color indexes */
	int white, ltgrey, dkgrey, black, red, yellow;
	int month, day, year, hour, minute;
	float dcvoltage, dccurrent, dcpower, x1, ya1, yb1, x2, ya2, yb2;
	int i;
	char s[32];
	char inputline[1000] = "";
	
	/* Allocate the image */
	im = gdImageCreate(527, 510);
	
	/* Allocate the color white (red, green, and blue all maximum).
	 Since this is the first color in a new image, it will
	 be the background color. */
	white = gdImageColorAllocate(im, 255, 255, 255);
	
	/* Allocate the color black (red, green, and blue all minimum). */
	black = gdImageColorAllocate(im, 0, 0, 0);
	
	ltgrey = gdImageColorAllocate(im, 170, 170, 170);
	dkgrey = gdImageColorAllocate(im, 85, 85, 85);
	red = gdImageColorAllocate(im, 255, 0, 0);
	yellow = gdImageColorAllocate(im, 255, 200, 0);
	
	/* Draw grey grid */
	for (i=0;i<23;i++) {
		gdImageLine(im, 40+20*i, 30, 40+20*i, 490, ltgrey);
		gdImageLine(im, 40+20*i, 486, 40+20*i, 490, black);
	}
	
	for (i=0;i<22;i++) {
		gdImageLine(im, 20, 50+20*i, 500, 50+20*i, ltgrey);
		gdImageLine(im, 20, 50+20*i, 24, 50+20*i, black);
	}
	
	/* Draw shadow */
	gdImageLine(im, 21, 491, 501, 491, dkgrey);
	gdImageLine(im, 501, 31, 501, 491, dkgrey);
	gdImageLine(im, 22, 492, 502, 492, ltgrey);
	gdImageLine(im, 502, 32, 502, 492, ltgrey);
	
	/* Label x-axis */
	for (i=1;i<=11;i++) {
		sprintf(s,"%d",i);
		gdImageString(im, gdFontGetSmall(), 20+20*i-(strlen(s)*gdFontGetSmall()->w/2), 494, s, black);
		gdImageString(im, gdFontGetSmall(), 260+20*i-(strlen(s)*gdFontGetSmall()->w/2), 494, s, black);
	}
//	strcpy(s,"noon");
	strcpy(s,"12");
	gdImageString(im, gdFontGetSmall(), 260-(strlen(s)*gdFontGetSmall()->w/2), 494, s, black);
	
	/* Label left y-axis (voltage) */
	for (i=1;490-i*80/((int) VOLTAGESCALE)-gdFontGetSmall()->h/2>40;i++) {
		sprintf(s,"%d",i+10*((int) VOLTAGESCALE));
		gdImageString(im, gdFontGetSmall(), 16-(strlen(s)*gdFontGetSmall()->w), 490-80/((int) VOLTAGESCALE)*i-gdFontGetSmall()->h/2, s, red);
	}
	
	/* Label right y-axis (power) */
	for (i=1;i<=22;i++) {
		sprintf(s,"%d",i*((int) (VOLTAGESCALE*20.0/POWERSCALE)));
		gdImageString(im, gdFontGetSmall(), 506, 490-20*i-gdFontGetSmall()->h/2, s, yellow);
	}
	
	/* Draw voltage label in red */
	strcpy(s,"Battery Voltage");
	gdImageString(im, gdFontGetSmall(), 20, 16, s, red);
	
	/* Draw Charging Power label for Charge Controller in yellow */
	strcpy(s,"Charging Power");
	gdImageString(im, gdFontGetSmall(), 500-(strlen(s)*gdFontGetSmall()->w), 16, s, yellow);
	
	/* Set clipping rectangle */
	gdImageSetClip(im, 20, 30, 500, 500);
	
	i=0;
	infile = fopen(logfilename, "r");
	if(infile != NULL) {
		while (fscanf(infile, "%[^\n]\n", inputline) != EOF)
		{
			sscanf(inputline,"%2d%*c%2d%*c%4d%2d%*c%2d%f%*f%f%*s",&month,&day,&year,&hour,&minute,&dcvoltage,&dccurrent);
			dcpower=dcvoltage*dccurrent;
			x2=(hour+minute/60.0)*20.0;
			ya2=(dcvoltage-10.0*VOLTAGESCALE)*80.0/VOLTAGESCALE;
			yb2=dcpower*POWERSCALE/VOLTAGESCALE;
			if (i < 1) {
				x1 = x2;
				ya1 = ya2;
				yb1 = yb2;
			}
			gdImageLine(im, 20+x1, 490-yb1, 20+x2, 490-yb2, yellow);
			gdImageLine(im, 20+x1, 490-ya1, 20+x2, 490-ya2, red);
			x1 = x2;
			ya1 = ya2;
			yb1 = yb2;
			i++;
		}
	}
	fclose(infile);
	
	/* Set clipping rectangle */
	gdImageSetClip(im, 0, 0, 527, 510);
	
	/* Frame graph */
	gdImageRectangle(im, 20, 30, 500, 490, black);
	
	/* Draw date at top of graph */
	sprintf(s,"%02d/%02d/%d",month,day,year);
	gdImageString(im, gdFontGetLarge(),im->sx / 2 - (strlen(s) * gdFontGetLarge()->w / 2), 12, s, black);
	
	/* Open a file for writing. "wb" means "write binary", important
	 under MSDOS, harmless under Unix. */
	pngout = fopen(graphfilename, "wb");
	
	/* Output the image to the disk file in PNG format. */
	gdImagePng(im, pngout);
	
	/* Close the files. */
	fclose(pngout);
	
	/* Destroy the image in memory. */
	gdImageDestroy(im);
}
