/*
 *  tristarCalibrationValues.c - This program reads all the Calibration Value registers on a Moringstar TriStar PWM and prints the results.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` tristarCalibrationValues.c -o tristarCalibrationValues */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int rc;
	char serial[9];
	float K_gain_adc_vb48, K_gain_adc_vb1224, K_gain_adc_ic, K_gain_adc_il, K_gain_adc_vs, K_gain_adc_vx;
	unsigned short K_hw_ver_major, K_hw_ver_minor, calib_state, K_amp60;
	uint16_t data[16];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the Calibration Value Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xF000, 12, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	printf("\nCalibration Value Registers\n\n");
	
	serial[0]=data[0] & 0x00FF;
	serial[1]=data[0] >> 8;
	serial[2]=data[1] & 0x00FF;
	serial[3]=data[1] >> 8;
	serial[4]=data[2] & 0x00FF;
	serial[5]=data[2] >> 8;
	serial[6]=data[3] & 0x00FF;
	serial[7]=data[3] >> 8;
	serial[8]='\0';
	printf("serial = %s\n",serial);
	
	K_gain_adc_vb48=data[4]/32768.0;
	printf("K_gain_adc_vb48 = %.2f\n",K_gain_adc_vb48);
	
	K_gain_adc_vb1224=data[5]/32768.0;
	printf("K_gain_adc_vb1224 = %.2f\n",K_gain_adc_vb1224);
	
	K_gain_adc_ic=data[6]/32768.0;
	printf("K_gain_adc_ic = %.2f\n",K_gain_adc_ic);
	
	K_gain_adc_il=data[7]/32768.0;
	printf("K_gain_adc_il = %.2f\n",K_gain_adc_il);
	
	K_gain_adc_vs=data[8]/32768.0;
	printf("K_gain_adc_vs = %.2f\n",K_gain_adc_vs);
	
	K_gain_adc_vx=data[9]/32768.0;
	printf("K_gain_adc_vx = %.2f\n",K_gain_adc_vx);
	
	K_hw_ver_minor=data[10] & 0x00FF;
	K_hw_ver_major=data[10] >> 8;
	printf("K_hw_ver = %d.%d\n",K_hw_ver_major, K_hw_ver_minor);
	
	K_amp60=data[11] & 0x00FF;
	if (K_amp60 == 0x00) {
		printf("K_amp60 = TS45\n");
	}
	if (K_amp60 == 0x01) {
		printf("K_amp60 = TS60\n");
	}
	
	calib_state=data[11] >> 8;
	if (calib_state == 0x5A) {
		printf("calib_state = Calibrated\n\n");
	}
	else {
		printf("calib_state = Not Calibrated\n\n");
	}

	/* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	return(0);
}

