/*
 *  tristarEEPROM.c - This program reads all the EEPROM registers on a Moringstar TriStar PWM and prints the results.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` tristarEEPROM.c -o tristarEEPROM */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int rc;
	float EV_reg, EV_float, EV_floatlb_trip, EV_float_cancel, EV_eq;
	unsigned short Et_float, Et_floatlb, Et_eqcalendar, Et_eq_above, Et_eq_reg;
	float EV_tempcomp, EV_hvd, EV_hvr;
	short Etmr_eqcalendar, Et_float_exit_cum, Ed_float_enter, Eb_diverison_pwm;
	float EV_lvd, EV_lvr, EV_lhvd, EV_lhvr, ER_icomp, Et_lvd_warn;
	float Et_sun1, Et_sun2;
	unsigned short Eb_light_config;
	float EV_night_max, EV_day_min, EV_relaxed_day;
	float EV_soc_g_gy, EV_soc_gy_y, EV_soc_y_yr, EV_soc_yr_r;
	short ETb_max, ETb_min;
	unsigned short Emb_tristar_id, Et_batt_service, Etmr_batt_service, EkWh, Emb_meterbus_id;
	unsigned int Ehourmeter;
	float EAh_r, EAh_t, EVb_min, EVb_max;
	uint16_t data[50];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE000, 15, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	printf("\nEEPROM Registers\n\n");
	
	printf("Charge and Diversion Mode Settings\n");
	
	EV_reg=data[0]*96.667/32768.0;
	printf("EV_reg = %.2f V\n",EV_reg);
	
	EV_float=data[1]*96.667/32768.0;
	printf("EV_float = %.2f V\n",EV_float);
	
	Et_float=data[2];
	printf("Et_float = %d s\n",Et_float);
	
	Et_floatlb=data[3];
	printf("Et_floatlb = %d s\n",Et_floatlb);
	
	EV_floatlb_trip=data[4]*96.667/32768.0;
	printf("EV_floatlb_trip = %.2f V\n",EV_floatlb_trip);
	
	EV_float_cancel=data[5]*96.667/32768.0;
	printf("EV_float_cancel = %.2f V\n",EV_float_cancel);
	
	EV_eq=data[6]*96.667/32768.0;
	printf("EV_eq = %.2f V\n",EV_eq);
	
	Et_eqcalendar=data[7];
	printf("Et_eqcalendar = %d days\n",Et_eqcalendar);
	
	Et_eq_above=data[8];
	printf("Et_eq_above = %d s\n",Et_eq_above);
	
	Et_eq_reg=data[9];
	printf("Et_eq_reg = %d s\n",Et_eq_reg);
	
	EV_tempcomp=data[10]*96.667/32768.0;
	printf("EV_tempcomp = %.2f V\n",EV_tempcomp);
	
	EV_hvd=data[11]*96.667/32768.0;
	printf("EV_hvd = %.2f V\n",EV_hvd);
	
	EV_hvr=data[12]*96.667/32768.0;
	printf("EV_hvr = %.2f V\n",EV_hvr);
	
	Etmr_eqcalendar=data[13];
	printf("Etmr_eqcalendar = %d days\n",Etmr_eqcalendar);
	
	Et_float_exit_cum=data[14];
	printf("Et_float_exit_cum = %d s\n",Et_float_exit_cum);
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
	
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE041, 2, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	Ed_float_enter=data[0];
	printf("Ed_float_enter = %d %%\n",Ed_float_enter);
	
	Eb_diverison_pwm=data[1];
	printf("Eb_diverison_pwm = %d\n",Eb_diverison_pwm);
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
		
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE00F, 21, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	printf("\nLoad Mode Settings\n\n");
	
	EV_lvd=data[0]*96.667/32768.0;
	printf("EV_lvd = %.2f V\n",EV_lvd);
	
	EV_lvr=data[1]*96.667/32768.0;
	printf("EV_lvr = %.2f V\n",EV_lvr);
	
	EV_lhvd=data[2]*96.667/32768.0;
	printf("EV_lhvd = %.2f V\n",EV_lhvd);
	
	EV_lhvr=data[3]*96.667/32768.0;
	printf("EV_lhvr = %.2f V\n",EV_lhvr);
	
	ER_icomp=data[4]*0.305/65536.0;
	printf("ER_icomp = %.2f ohms\n",ER_icomp);
	
	Et_lvd_warn=data[5]*0.1;
	printf("Et_lvd_warn = %.2f s\n",Et_lvd_warn);
	
	printf("\nLighting Mode Settings\n\n");
	
	Et_sun1=data[6];
	printf("Et_sun1 = %d s\n",Et_sun1);
	
	Et_sun2=data[7];
	printf("Et_sun2 = %d s\n",Et_sun2);
	
	Eb_light_config=data[8];
//	printf("Eb_light_config = %d s\n",Eb_light_config);
	
	printf("Eb_light_config = lighting configuration bits settings:\n");
	if (Eb_light_config & 1) {
		printf("\tBit 0 ON - Load is turned ON at DUSK\n");
	} else {
		printf("\tBit 0 OFF\n");
	}
	if ((Eb_light_config & (1 << 1)) >> 1) {
		printf("\tBit 1 ON - Load is turned OFF at DUSK\n");
	} else {
		printf("\tBit 1 OFF\n");
	}
	if ((Eb_light_config & (1 << 2)) >> 2) {
		printf("\tBit 2 ON - Et_sun1 times with respect to DUSK\n");
	} else {
		printf("\tBit 2 OFF - Et_sun1 times with respect to DAWN\n");
	}
	if ((Eb_light_config & (1 << 3)) >> 3) {
		printf("\tBit 3 ON - Et_sun2 times with respect to DAWN\n");
	} else {
		printf("\tBit 3 OFF - Et_sun2 times with respect to DUSK\n");
	}
	
	EV_night_max=data[9]*139.15/32768.0;
	printf("EV_night_max = %.2f V\n",EV_night_max);
	
	EV_day_min=data[10]*139.15/32768.0;
	printf("EV_day_min = %.2f V\n",EV_day_min);
	
	EV_relaxed_day=data[11]*139.15/32768.0;
	printf("EV_relaxed_day = %.2f V\n",EV_relaxed_day);
	
	printf("\nCommon Values\n\n");
	
	EV_soc_g_gy=data[12]*96.667/32768.0;
	printf("EV_soc_g_gy = %.2f V\n",EV_soc_g_gy);
	
	EV_soc_gy_y=data[13]*96.667/32768.0;
	printf("EV_soc_gy_y = %.2f V\n",EV_soc_gy_y);
	
	EV_soc_y_yr=data[14]*96.667/32768.0;
	printf("EV_soc_y_yr = %.2f V\n",EV_soc_y_yr);
	
	EV_soc_yr_r=data[15]*96.667/32768.0;
	printf("EV_soc_yr_r = %.2f V\n",EV_soc_yr_r);
	
	ETb_max=data[16];
	printf("ETb_max = %d °C\n",ETb_max);
	
	ETb_min=data[17];
	printf("ETb_min = %d °C\n",ETb_min);
	
	Emb_tristar_id=data[18];
	printf("Emb_tristar_id = %d\n",Emb_tristar_id);
	
	Et_batt_service=data[19];
	printf("Et_batt_service = %d days\n",Et_batt_service);
	
	Etmr_batt_service=data[20];
	printf("Etmr_batt_service = %d days\n",Etmr_batt_service);
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
	
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE026, 9, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	Ehourmeter=(data[1] << 16) + data[0];
	printf("Ehourmeter = %d h\n",Ehourmeter);
	
	EAh_r=((data[3] << 16) + data[2])*0.1;
	printf("EAh_r = %.2f Ah\n",EAh_r);
	
	EAh_t=((data[5] << 16) + data[4])*0.1;
	printf("EAh_t = %.2f Ah\n",EAh_t);
	
	EkWh=data[6];
	printf("EkWh = %d kWh\n",EkWh);
	
	EVb_min=data[7]*96.667/32768.0;
	printf("EVb_min = %.2f V\n",EVb_min);
	
	EVb_max=data[8]*96.667/32768.0;
	printf("EVb_max = %.2f V\n",EVb_max);
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
	
	/* Read the EEPROM Registers and convert the results to their proper values */
	rc = modbus_read_registers(ctx, 0xE040, 1, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
		
	Emb_meterbus_id=data[0];
	printf("Emb_meterbus_id = %d\n\n",Emb_meterbus_id);
	
	/* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	return(0);
}

