/*
 *  tristarRAM.c - This program reads all the RAM registers on a Moringstar TriStar PWM and prints the results.
 *  

Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` tristarRAM.c -o tristarRAM */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int rc;
	float adc_vb_f,adc_vs_f,adc_vx_f,adc_ipv_f,adc_iload_f, Vb_f, V_ref, Ahc_r, Ahc_t;
	short T_hs, T_batt;
	unsigned short fault, dip_switch, d_filt, control_mode, control_state;
	unsigned int hourmeter, alarm;
	uint16_t data[50];
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the RAM Registers */
	rc = modbus_read_registers(ctx, 0x0008, 22, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	/* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	/* Convert the results to their proper values and print them out */
	printf("\nRAM Registers\n\n");
	
	adc_vb_f=data[0]*96.667/32768.0;
	printf("adc_vb_f = %.2f V\n",adc_vb_f);
	
	adc_vs_f=data[1]*96.667/32768.0;
	printf("adc_vs_f = %.2f V\n",adc_vs_f);
	
	adc_vx_f=data[2]*139.15/32768.0;
	printf("adc_vx_f = %.2f V\n",adc_vx_f);
	
	adc_ipv_f=data[3]*66.667/32768.0;
	printf("adc_ipv_f = %.2f A\n",adc_ipv_f);
	
	adc_iload_f=data[4]*316.67/32768.0;
	printf("adc_iload_f = %.2f A\n",adc_iload_f);
	
	Vb_f=data[5]*96.667/32768.0;
	printf("Vb_f = %.2f V\n",Vb_f);
	
	T_hs=data[6];
	printf("T_hs = %d °C\n",T_hs);
	
	T_batt=data[7];
	if (T_batt == 0x80) {
		printf("RTS Not Connected\n");
	} else {
		printf("T_batt = %d °C\n",T_batt);
	}
	
	V_ref=data[8]*96.667/32768.0;
	printf("V_ref = %.2f V\n",V_ref);
		
	Ahc_r=((data[9] << 16) + data[10])*0.1;
	printf("Ahc_r = %.2f Ah\n",Ahc_r);
	
	Ahc_t=((data[11] << 16) + data[12])*0.1;
	printf("Ahc_t = %.2f Ah\n",Ahc_t);
	
	hourmeter=(data[13] << 16) + data[14];
	printf("hourmeter = %d h\n",hourmeter);
	
	alarm=(data[21] << 16) + data[15];
	printf("alarm = Controller self-diagnostic alarms:\n");
	if (alarm == 0) {
		printf("\tNo alarms\n");
	} else {
		if (alarm & 1) printf("\tRTS open\n");
		if ((alarm & (1 << 1)) >> 1) printf("\tRTS shorted\n");
		if ((alarm & (1 << 2)) >> 2) printf("\tRTS disconnected\n");
		if ((alarm & (1 << 3)) >> 3) printf("\tThs disconnected\n");
		if ((alarm & (1 << 4)) >> 4) printf("\tThs shorted\n");
		if ((alarm & (1 << 5)) >> 5) printf("\tTriStar hot\n");
		if ((alarm & (1 << 6)) >> 6) printf("\tCurrent limit\n");
		if ((alarm & (1 << 7)) >> 7) printf("\tCurrent offset\n");
		if ((alarm & (1 << 8)) >> 8) printf("\tBattery Sense\n");
		if ((alarm & (1 << 9)) >> 9) printf("\tBatt Sense disc\n");
		if ((alarm & (1 << 10)) >> 10) printf("\tUncalibrated\n");
		if ((alarm & (1 << 11)) >> 11) printf("\tRTS miswire\n");
		if ((alarm & (1 << 12)) >> 12) printf("\tHVD\n");
		if ((alarm & (1 << 13)) >> 13) printf("\thigh d\n");
		if ((alarm & (1 << 14)) >> 14) printf("\tMiswire\n");
		if ((alarm & (1 << 15)) >> 15) printf("\tFET open\n");
		if ((alarm & (1 << 16)) >> 16) printf("\tP12\n");
		if ((alarm & (1 << 17)) >> 17) printf("\tLoad Disc.\n");
		if ((alarm & (1 << 18)) >> 18) printf("\tAlarm 19\n");
		if ((alarm & (1 << 19)) >> 19) printf("\tAlarm 20\n");
		if ((alarm & (1 << 20)) >> 20) printf("\tAlarm 21\n");
		if ((alarm & (1 << 21)) >> 21) printf("\tAlarm 22\n");
		if ((alarm & (1 << 22)) >> 22) printf("\tAlarm 23\n");
		if ((alarm & (1 << 23)) >> 23) printf("\tAlarm 24\n");
	}
	
	fault=data[16];
	printf("fault = Solar input self-diagnostic faults:\n");
	if (fault == 0) {
		printf("\tNo faults\n");
	} else {
		if (fault & 1) printf("\tExternal Short Circuit\n");
		if ((fault & (1 << 1)) >> 1) printf("\tOvercurrent\n");
		if ((fault & (1 << 2)) >> 2) printf("\tFETs shorted\n");
		if ((fault & (1 << 3)) >> 3) printf("\tSoftware bug\n");
		if ((fault & (1 << 4)) >> 4) printf("\tHVD\n");
		if ((fault & (1 << 5)) >> 5) printf("\tTristar Hot\n");
		if ((fault & (1 << 6)) >> 6) printf("\tDIP sw changed\n");
		if ((fault & (1 << 7)) >> 7) printf("\tSetting edit\n");
		if ((fault & (1 << 8)) >> 8) printf("\tReset?\n");
		if ((fault & (1 << 9)) >> 9) printf("\tMiswire\n");
		if ((fault & (1 << 10)) >> 10) printf("\tRTS shorted\n");
		if ((fault & (1 << 11)) >> 11) printf("\tRTS disconnected\n");
		if ((fault & (1 << 12)) >> 12) printf("\tFault 12\n");
		if ((fault & (1 << 13)) >> 13) printf("\tFault 13\n");
		if ((fault & (1 << 14)) >> 14) printf("\tFault 14\n");
		if ((fault & (1 << 15)) >> 15) printf("\tFault 15\n");
	}
	
	dip_switch=data[17];
	printf("dip_switch = DIP switch settings:\n");
	if (dip_switch & 1) {
		printf("\tSwitch 1 ON\n");
	} else {
		printf("\tSwitch 1 OFF\n");
	}
	if ((dip_switch & (1 << 1)) >> 1) {
		printf("\tSwitch 2 ON\n");
	} else {
		printf("\tSwitch 2 OFF\n");
	}
	if ((dip_switch & (1 << 2)) >> 2) {
		printf("\tSwitch 3 ON\n");
	} else {
		printf("\tSwitch 3 OFF\n");
	}
	if ((dip_switch & (1 << 3)) >> 3) {
		printf("\tSwitch 4 ON\n");
	} else {
		printf("\tSwitch 4 OFF\n");
	}
	if ((dip_switch & (1 << 4)) >> 4) {
		printf("\tSwitch 5 ON\n");
	} else {
		printf("\tSwitch 5 OFF\n");
	}
	if ((dip_switch & (1 << 5)) >> 5) {
		printf("\tSwitch 6 ON\n");
	} else {
		printf("\tSwitch 6 OFF\n");
	}
	if ((dip_switch & (1 << 6)) >> 6) {
		printf("\tSwitch 7 ON\n");
	} else {
		printf("\tSwitch 7 OFF\n");
	}
	if ((dip_switch & (1 << 7)) >> 7) {
		printf("\tSwitch 8 ON\n");
	} else {
		printf("\tSwitch 8 OFF\n");
	}
	
	control_mode=data[18];
	switch (control_mode) {
		case 0x00:
			printf("control_mode = 0x%0X Charge\n",control_mode);
			break;
		case 0x01:
			printf("control_mode = 0x%0X Load\n",control_mode);
			break;
		case 0x02:
			printf("control_mode = 0x%0X Diversion\n",control_mode);
			break;
		case 0x03:
			printf("control_mode = 0x%0X Lighting\n",control_mode);
			break;
	}
	
	control_state=data[19];
	if (control_mode == 0x00 || control_mode == 0x02) {		/* Charge and Diversion Modes */
		switch (control_state) {
			case 0:
				printf("control_state = %d START\n",control_state);
				break;
			case 1:
				printf("control_state = %d NIGHT_CHECK\n",control_state);
				break;
			case 2:
				printf("control_state = %d DISCONNECT\n",control_state);
				break;
			case 3:
				printf("control_state = %d NIGHT\n",control_state);
				break;
			case 4:
				printf("control_state = %d FAULT\n",control_state);
				break;
			case 5:
				printf("control_state = %d BULK\n",control_state);
				break;
			case 6:
				printf("control_state = %d PWM\n",control_state);
				break;
			case 7:
				printf("control_state = %d FLOAT\n",control_state);
				break;
			case 8:
				printf("control_state = %d EQUALIZE\n",control_state);
				break;
		}
	}
	if (control_mode == 0x01 || control_mode == 0x03) {		/* Load and Lighting Modes */
		switch (control_state) {
			case 0:
				printf("control_state = %d START\n",control_state);
				break;
			case 1:
				printf("control_state = %d NORMAL\n",control_state);
				break;
			case 2:
				printf("control_state = %d LVD_WARN\n",control_state);
				break;
			case 3:
				printf("control_state = %d LVD\n",control_state);
				break;
			case 4:
				printf("control_state = %d FAULT\n",control_state);
				break;
			case 5:
				printf("control_state = %d DISCONNECT\n",control_state);
				break;
			case 6:
				printf("control_state = %d NORMAL_OFF\n",control_state);
				break;
			case 7:
				printf("control_state = %d OVERRIDE_LVD\n",control_state);
				break;
		}
	}
	
	d_filt=data[20];
	printf("d_filt = %.1f %% - PWM Duty Cycle\n\n",100.0*d_filt/230.0);
	
	return(0);
}

