/*
 *  tristarlog.c - This program reads all the log registers from a Moringstar TriStar PWM, sorts the records by hourmeter, and prints the results.
 *  
 *	Note: The Morningstar TriStar PWM creates a new log entry when the charge state switches to night.  This program reads the
 *		  charge state from RAM to create the proper date stamps.
 *
 
Copyright 2014 Tom Rinehart.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


/* Compile with: cc `pkg-config --cflags --libs libmodbus` tristarlog.c -o tristarlog */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <modbus.h>

#include "powersystem.h"

int main(void)
{
	modbus_t *ctx;
	int i, j, k, n, low, rc, hour, today, indx[90];
	unsigned int hm, hourmeter[90], Logger_flags[90], alarm_daily[90];
	float adc_ipv_f, Vb_min_daily[90], Vb_max_daily[90], Ah_daily[90];
	unsigned short Fault_daily[90], Tb_min_daily[90], Tb_max_daily[90], T_ab_daily[90], T_eq_daily[90], T_fl_daily[90];
	unsigned short control_state;
	unsigned short data[30];
	time_t lclTime, logTime;
	struct tm *now,*logthen;
	char tsdate[32], tshour[3];
	
	/* Get current local time */
	lclTime = time(NULL);
	now = localtime(&lclTime);
	
	/* Set up a new MODBUS context */
	ctx = modbus_new_rtu(SERIALPORTPATH, 9600, 'N', 8, 2);
	if (ctx == NULL) {
		fprintf(stderr, "Unable to create the libmodbus context\n");
		return -1;
	}
	
	/* Set the slave id to the TriStar PWM MODBUS id */
	modbus_set_slave(ctx, TRISTARPWM);
	
	/* Open the MODBUS connection to the TriStar PWM */
    if (modbus_connect(ctx) == -1) {
        fprintf(stderr, "Connection failed TRISTARPWM: %s\n", modbus_strerror(errno));
        modbus_free(ctx);
        return -1;
    }
	
	/* Read the RAM Registers */
	rc = modbus_read_registers(ctx, 0x0008, 22, data);
	if (rc == -1) {
		fprintf(stderr, "%s\n", modbus_strerror(errno));
		return -1;
	}
	
	/* Determine if the last daily log record is from today or yesterday */
	strftime(tshour, 3, "%H", now);
	hour=atoi(tshour);
	adc_ipv_f=data[3]*66.667/32768.0;
	control_state=data[19];
	if ((control_state == 3) && (hour > 12) && (adc_ipv_f < 0.001)) {		/* If the charge state is night, the time is past noon, 
																			 and the array current is < 0.001, then the newest log record is from today */
		today=1;
	}
	else {											/* Otherwise, the newest record is from yesterday */
		today=0;
	}
	
	usleep(2500);		// Give the charge controller time before requesting next set of registers
	
	j=0;
	for(i=0x8000; i<0x8600; i+=0x0010) {
		
		/* Read the log registers and convert the results to their proper values */
		rc = modbus_read_registers(ctx, i, 12, data);
		if (rc == -1) {
			fprintf(stderr, "modbus_read_registers: %s\n", modbus_strerror(errno));
			return -1;
		}
		
		hm=data[0] + ((data[1] & 0x00FF) << 16);
		if (hm != 0x000000 && hm != 0xFFFFFF) {
			hourmeter[j]=hm;
			
			Logger_flags[j]=((data[2] & 0x00FF) << 8) + (data[1] >> 8);
			
			alarm_daily[j]=(data[3] << 8) + (data[2] >> 8);
			
			Vb_max_daily[j]=data[4]*100.0/32768.0;
			
			Vb_min_daily[j]=data[5]*100.0/32768.0;
			
			Ah_daily[j]=data[6]*0.1;
			
			Fault_daily[j]=data[7];
			
			Tb_min_daily[j]=(data[8] & 0x00FF);
			
			Tb_max_daily[j]=(data[8] >> 8);
			
			T_ab_daily[j]=data[9];
			
			T_eq_daily[j]=data[10];
			
			T_fl_daily[j]=data[11];
			
			j++;
		}
		usleep(2500);					// Give the charge controller time before requesting next set of log registers
	}
	
    /* Close the MODBUS connection */
    modbus_close(ctx);
    modbus_free(ctx);
	
	/* Order the data with lowest hourmeter value first */
	n=j;
	low=0;
	for (i=0; i<n-1; i++) {
		if (hourmeter[i] > hourmeter[i+1]) {		//Search for the lowest hourmeter value
			low=i+1;
			break;
		}
	}
	if (low > 0) {									// Create an index to order the data
		for (i=0, j=n-low; i<low; i++, j++) indx[j]=i;
		for (i=low, j=0; i<n; i++, j++) indx[j]=i;
	}
	
	/* Print out each of the daily log records */
	for (i=0; i<n; i++) {
		if (i != 0) {
			printf("-------------------------------------------------------------\n\n");
		}
		logTime=lclTime-(n-i-today)*(24*60*60);
		logthen = localtime(&logTime);
		strftime(tsdate, 32, "%m/%d/%Y", logthen);
		printf("Date = %s\n",tsdate);
		printf("hourmeter = %d h\n",hourmeter[indx[i]]);
		printf("Logger_flags = %d h\n",Logger_flags[indx[i]]);
		printf("alarm_daily = Daily controller self-diagnostic alarms:\n");
		if (alarm_daily[indx[i]] == 0) {
			printf("\tNo alarms\n");
		} else {
			if (alarm_daily[indx[i]] & 1) printf("\tRTS open\n");
			if ((alarm_daily[indx[i]] & (1 << 1)) >> 1) printf("\tRTS shorted\n");
			if ((alarm_daily[indx[i]] & (1 << 2)) >> 2) printf("\tRTS disconnected\n");
			if ((alarm_daily[indx[i]] & (1 << 3)) >> 3) printf("\tThs disconnected\n");
			if ((alarm_daily[indx[i]] & (1 << 4)) >> 4) printf("\tThs shorted\n");
			if ((alarm_daily[indx[i]] & (1 << 5)) >> 5) printf("\tTriStar hot\n");
			if ((alarm_daily[indx[i]] & (1 << 6)) >> 6) printf("\tCurrent limit\n");
			if ((alarm_daily[indx[i]] & (1 << 7)) >> 7) printf("\tCurrent offset\n");
			if ((alarm_daily[indx[i]] & (1 << 8)) >> 8) printf("\tBattery Sense\n");
			if ((alarm_daily[indx[i]] & (1 << 9)) >> 9) printf("\tBatt Sense disc\n");
			if ((alarm_daily[indx[i]] & (1 << 10)) >> 10) printf("\tUncalibrated\n");
			if ((alarm_daily[indx[i]] & (1 << 11)) >> 11) printf("\tRTS miswire\n");
			if ((alarm_daily[indx[i]] & (1 << 12)) >> 12) printf("\tHVD\n");
			if ((alarm_daily[indx[i]] & (1 << 13)) >> 13) printf("\thigh d\n");
			if ((alarm_daily[indx[i]] & (1 << 14)) >> 14) printf("\tmiswire\n");
			if ((alarm_daily[indx[i]] & (1 << 15)) >> 15) printf("\tFET open\n");
			if ((alarm_daily[indx[i]] & (1 << 16)) >> 16) printf("\tP12\n");
			if ((alarm_daily[indx[i]] & (1 << 17)) >> 17) printf("\tLoad Disc.\n");
			if ((alarm_daily[indx[i]] & (1 << 18)) >> 18) printf("\tAlarm 19\n");
			if ((alarm_daily[indx[i]] & (1 << 19)) >> 19) printf("\tAlarm 20\n");
			if ((alarm_daily[indx[i]] & (1 << 20)) >> 20) printf("\tAlarm 21\n");
			if ((alarm_daily[indx[i]] & (1 << 21)) >> 21) printf("\tAlarm 22\n");
			if ((alarm_daily[indx[i]] & (1 << 22)) >> 22) printf("\tAlarm 23\n");
			if ((alarm_daily[indx[i]] & (1 << 23)) >> 23) printf("\tAlarm 24\n");
		}
		printf("Vb_max_daily = %.2f V\n",Vb_max_daily[indx[i]]);
		printf("Vb_min_daily = %.2f V\n",Vb_min_daily[indx[i]]);
		printf("Ah_daily = %.2f Ah\n",Ah_daily[indx[i]]);
		printf("Fault_daily = Daily controller self-diagnostic faults:\n");
		if (Fault_daily[indx[i]] == 0) {
			printf("\tNo faults\n");
		} else {
			if (Fault_daily[indx[i]] & 1) printf("\tExternal Short\n");
			if ((Fault_daily[indx[i]] & (1 << 1)) >> 1) printf("\tOvercurrent\n");
			if ((Fault_daily[indx[i]] & (1 << 2)) >> 2) printf("\tFET short\n");
			if ((Fault_daily[indx[i]] & (1 << 3)) >> 3) printf("\tSoftware\n");
			if ((Fault_daily[indx[i]] & (1 << 4)) >> 4) printf("\tHVD\n");
			if ((Fault_daily[indx[i]] & (1 << 5)) >> 5) printf("\tTriStar hot\n");
			if ((Fault_daily[indx[i]] & (1 << 6)) >> 6) printf("\tDIP sw changed\n");
			if ((Fault_daily[indx[i]] & (1 << 7)) >> 7) printf("\tSetting edit\n");
			if ((Fault_daily[indx[i]] & (1 << 8)) >> 8) printf("\treset?\n");
			if ((Fault_daily[indx[i]] & (1 << 9)) >> 9) printf("\tMiswire\n");
			if ((Fault_daily[indx[i]] & (1 << 10)) >> 10) printf("\tRTS shorted\n");
			if ((Fault_daily[indx[i]] & (1 << 11)) >> 11) printf("\tRTS disconnected\n");
			if ((Fault_daily[indx[i]] & (1 << 12)) >> 12) printf("\tFault 12\n");
			if ((Fault_daily[indx[i]] & (1 << 13)) >> 13) printf("\tFault 13\n");
			if ((Fault_daily[indx[i]] & (1 << 14)) >> 14) printf("\tFault 14\n");
			if ((Fault_daily[indx[i]] & (1 << 15)) >> 15) printf("\tFault 15\n");
		}
		printf("Tb_min_daily = %d °C\n",Tb_min_daily[indx[i]]);
		printf("Tb_max_daily = %d °C\n",Tb_max_daily[indx[i]]);
		printf("T_ab_daily = %d min\n",T_ab_daily[indx[i]]);
		printf("T_eq_daily = %d min\n",T_eq_daily[indx[i]]);
		printf("T_fl_daily = %d min\n\n",T_fl_daily[indx[i]]);
	}
	
	return(0);
}

